
    // Character.hpp
    // Copyright (C) 2013  Giménez, Christian N.

    // This program is free software: you can redistribute it and/or modify
    // it under the terms of the GNU General Public License as published by
    // the Free Software Foundation, either version 3 of the License, or
    // (at your option) any later version.

    // This program is distributed in the hope that it will be useful,
    // but WITHOUT ANY WARRANTY; without even the implied warranty of
    // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    // GNU General Public License for more details.

    // You should have received a copy of the GNU General Public License
    // along with this program.  If not, see <http://www.gnu.org/licenses/>.

    // Viernes 26 De Abril Del 2013    


#ifndef __CHARACTER_HPP__
#define __CHARACTER_HPP__

#include <string>

using namespace std;

#include <OgreSceneManager.h>
#include <OgreSceneNode.h>
#include <OgreEntity.h>

#include "Location.hpp"
#include "Terminal.hpp"

using namespace Terminal;

#define CHARACTER_PATH "./models/characters"

/**
   I represent a Character, no matter if it is an NPC or PC.

   This is an @e abstract class.
 */
class Character{
  
protected:
  /**
     @name Ogre Specific Atributes
  */
  //@{
  Ogre::Entity *entity;
  Ogre::SceneNode *scene_node;
  //@}

  /**
     The entity name.
   */
  string ent_name;
  /**
     This is the model name. Also is used as a filename, so ensure that it has the file extension.
     For example: "joanmodel". 
   */
  string model_name;
  
  /**
     This is the location where I am.
   */
  Location *location;

  /*
     Position inside the Location. 

     This position is the "Camera" position of the character.

     By default is (0,0,0).
   
     vector3df position;*/

  /**
     Step length. This is used when I want to move forward, backwards; strafe left or right.
   */
  float step;
  
  /**
     Angle of rotation. Where I am looking to?
     In radians.
   */
  float angle;


  /**
     The model could be created facing another direction than the front.
     This can correct the angle of the direction of the model.
   */
  float initial_angle;


  /**
     Create an instance of Character with a model and situated in a Location.

     Every constructor of this class is used for inheritance purposes. Remember, this is an @em abstract class.
     
     @param name the name of the character(it must be a unique name because it will be used as the Ogre::Entity name).
     @param model_name is an instance of string that has the name of the model. Is used as a filename so ensure it has extension(For example: "joanmodel.3ds").
     @param location is an instance of Location where the model must be situated.
     @param initial_angle is the angle to add to the initial direction of the model. See the initial_angle atribute.
   */
  Character(string name, string model_name, Location *location, float initial_angle = 0);
  /**
     Create an instance of Character according that is positioned according to a parent.
     
     If parent is @c NULL then this behave as Character(string model_name, Location*location) constructor.

     Use @em addedpos @em addedrot and @em addedscale for moving around the parent position.

     Every constructor of this class is used for inheritance purposes. Remember, this is an @em abstract class.
 
     @param name the name of the character(it must be a unique name because it will be used as the Ogre::Entity name).    
     @param model_name is an instance of string that has the name of the model. Is used as a filename so ensure it has extension(For example: "joanmodel.3ds").
     @param location is an instance of Location where the model must be situated.
     @param parent is a Ogre::SceneNode(for example Ogre::Camera) that is the relative position.
   */
  Character(Ogre::SceneNode *parent, string name, string model_name, Location *location);
  
  /**
     @em Warning: The collection of animations @bf will be deleted!

     This means that you may need to use get_col_animation() method to retrieve the collection and copy it!
   */
  ~Character();

public:
  
  /**
     Load the character. 

     The collection of animations will be ready for use.

     @param smgr An instance of Ogre::SceneManager.
   */
  virtual void load(Ogre::SceneManager *smgr);


  /**
     @name Getters and Setters.     
  */
  //@{
  float* get_position();

  /**
     Set the position inside the location. 
   */
  void set_position(float pos[3]);

  Location* get_location();
  void set_location(Location *loc);


  float get_angle();
  void set_angle(float newangle);

  float get_initial_angle();

  float get_step();
  void set_step(float newstep);

  string get_name();  

  //@}

  /**
     @name Movement

     Functions for moving the player.
  */
  //@{
  /**
     I move forward one step!
   */
  virtual void forward();
  /**
     I move backwards one step!
  */
  virtual void backward();
  /**
     I strafe left one step!
  */
  virtual void left();
  /**
     I strafe right one step!
  */
  virtual void right();    

  /**
     Rotate the character to the left.
   */
  virtual void rot_left();
  /**
     Rotate the character to the right.
   */
  virtual void rot_right();

  //@}

  void play_animation(string name);

  void stop_animation();
};

#endif
