/*
  WorldFile.hpp
  Copyright (C) 2013  Giménez, Christian N.
  
  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  
  sábado 26 de enero del 2013     */


#ifndef __WORLDFILE_HPP
#define __WORLDFILE_HPP 1

#include <string>
#include <iostream>
#include <fstream>

using namespace std;

#include "WorldBuilder.hpp"

/*
  Constants:
*/

#define WF_ENDFILE "End"
#define WF_NAME "name"
#define WF_LOCATIONS "locations"

#define WF_PATH "../worlds/"

namespace Loaders{

  /**
     I'm a WorldFile class. Each of my instances can process a file with the "world file" syntax.
     
     You can set me a WorldBuilder for creating the World while I'm reading the file. Use the method load_file() in that case:

     @code

     ...

     WorldBuilder builder;
     
     WorldFile f(&builder, "/my/path/to/world/file");

     f.open_read();
     f.load_file();
     f.close();

     World *w = builder.get_world();

     ...
     
     @endcode

     @section World File Syntax
     
     @verbatim
     Begin name
     the world name
     initial location name
     Begin locations
     location 1
     location 2
     location 3
     location 4
     ...
     End
     @endverbatim     

     The initial location is another new location. It should not be listed in the @c locations section.    
  */
  class WorldFile {
    
  protected:
    /**
       This is the my filename (and maybe the path also, like @c /home/myuser/myfile).

       If no path is given, I complete it with the defined in the @c WF_PATH in WorldFile.hpp.
     */
    string filename;

    ifstream file;

    string next_type;
    
    /**
       Has the file been opened?
     */
    bool opened;

    /**
       A builder class for creating the World instance.
     */
    WorldBuilder *builder;

    /**
       @name Buffered Information of the World
       These are information for the world. I use this as a buffer when loading it.
    */
    //@{
    string name;

    string initial_location;
    //@}

  public:
    /**
       Create an instance of WorldFile for loading a World instance.

       I create the filename according the worldname.

       @todo Make checks for completing the filename with the path if no path is given.

       @param builder An instance of WorldBuilder instance for building the World with all the proper data.
       @param filename An instance of string for locating the world file.
     */
    WorldFile(WorldBuilder* builder, string filename);
    ~WorldFile();
    

    /**
       I load all the file using the builder and pass everything needed so a World can be created(if the file is in good shape!).

       A WorldBuilder must be setted to use this message.
     */
    void load_file();
    
    /**
       Retrieve the next type and load it. Use the getters in the "Retrieving Individual Information" section for getting the information. 

       If you passed a builder, you don't need anything, I'll tell the buider to do everything for you.

       @return An instance of string with the next element loaded. See the WF_* defines in the WorldFile.hpp file.
       @retval WF_ENDFILE if the file was not opened or reached the EOF.
       @retval "" An empty string means something goes wrong!.
     */
    string next();

    /**
       I'll open the file just for reading it.
    */
    void open_read();

    /**
       I'll close the file for you.
     */
    void close();

    /**
       @name Retrieving Individual Information 
    */
    //@{
    /**
       Retrieving the name after loading it.
       
       First use load() for getting the information. Ensure to use next() until it returns the "name" string or see WF_NAME define.
       
       @returns A string with the world name.
    */
    string get_name();    
    //@}

    

  protected:

    /**
       @name Loading the File

       This protected messages load each element of the file separatelly because each element has its own format.
       Getting strings, or floating coordinates numbers, etc. are part of this @c load_* functions according to the item I want to get from the file.
     */
    //@{
    /**
       I load the name(or I'll suppose it is the name).       
     */
    void load_name();
    
    /**
       Used inside the next() method, with this message I can process the next_type string for getting the next thing.

       I fill the next_type according the syntax of the world file.

       The element loaded is sended to the builder for creating the world. We use the @c build_* message depending on the type I'm trying to load.
     */
    void load_element();

    /**
       @todo Implement this feature
    */
    void load_locations();
    
    /**
       Load each location from the locations section and add send it directly to the builder. 

       This message is a bit faster and does not generate any vector information.
     */
    void load_locations_with_builder();

    void load_initial_location();
    //@}
    
  };
}

#endif /* __WORLDFILE_HPP */

